/* ------------------------------------------------------------- */
/* qagutil.h, qagutil.cc                                         */
/* Stefan Bauberger (stefan@bauberger.de)                        */
/* last revision 4.12.95                                         */
/* ------------------------------------------------------------- */
/* Interface for qag.cc,qag.h (adaptive gauss-kronrod-           */
/* integration) to integrate complex functions of real variables */
/* Cplx foo(Double x) in an interval or from a>=0 to infinity    */
/* ------------------------------------------------------------- */

// integration of \int_a^b f(x), with maximal relative error epsrel,
// the estimated achieved absolute error is return in *error:
//
// Cplx Cqag(Cplx (*f) (Double),Double a,Double b,Double epsrel,Double *error);

// integration of \int_a^b f(x) with better convergence in case
// of a sqrt-singularity at f(x=a):
//
// Cplx Cqagasqrtsing(Cplx (*f) (Double),Double a,Double b,
//                    Double epsrel,Double *error);

// integration of \int_a^\infty f(x):
//
// Cplx Cqaginf(Cplx (*f) (Double),Double a,
//              Double epsrel,Double *error); // integr. from a to infinity

// integration of \int_a^\infty f(x) with better convergence in case
// of a sqrt-singularity at f(x=a):
//
// Cplx Cqaginfasqrtsing(Cplx (*f) (Double),Double a,
//                       Double epsrel,Double *error);

// integration of \int_a^\infty f(x)/(x-singe-I*iepssign)
// iepssign = +/-1
//
// Cplx Cqagpinf(Cplx (*f) (Double),Double a,Double singe,int iepssign,
//               Double epsrel,Double *error);

// like previous Cqagpinf but take limit1 and limit2 as borders of
// integration intervals if larger than a
// Cplx Cqagpinf(Cplx (*f) (Double),Double a,Double singe,int iepssign,
//               Double epsrel,Double *error,Double limit1,Double limit2)

// like Cqagpinf but with better convergence in case
// of a sqrt-singularity at f(x=a):
// Cplx Cqagpinfasqrtsing(Cplx (*f) (Double),Double a,Double singe,int iepssign,
//                        Double epsabs,Double *error);

// integration of f(s)/(s-singe-iepssign*I*eps)^2 to infinity
// Df(x) is supposed to be the derivative of f(x) 
// Cplx Cqagp2inf(Cplx (*f) (Double),Cplx (*Df) (Double),
//                Double a,Double singe,int iepssign,
//                Double epsrel,Double *error);

#include "Cplx.h"

Cplx Cqag(Cplx (*f) (Double),Double a,Double b,Double epsrel,Double *error);

Cplx Cqagasqrtsing(Cplx (*f) (Double),Double a,Double b,
                   Double epsrel,Double *error);
Cplx Cqagasqrtsing2(Cplx (*f) (Double),Double a,Double b,
                    Double epsrel,Double *error);
// the same with better convergence in case of a sqrt-singularity at a

Cplx Cqaginf(Cplx (*f) (Double),Double a,
             Double epsrel,Double *error); // integr. from a to infinity
Cplx Cqaginf2(Cplx (*f) (Double),Double a,
             Double epsrel,Double *error); // integr. from a to infinity

Cplx Cqaginfasqrtsing(Cplx (*f) (Double),Double a,
                      Double epsrel,Double *error);
Cplx Cqaginfasqrtsing2(Cplx (*f) (Double),Double a,
                       Double epsrel,Double *error);
// the same with better convergence in case of a sqrt-singularity at a

Cplx Cqagpinf(Cplx (*f) (Double),Double a,Double singe,int iepssign,
              Double epsrel,Double *error);
Cplx Cqagpinf2(Cplx (*f) (Double),Double a,Double singe,int iepssign,
               Double epsrel,Double *error);
// integration of f(x)/(x-singe-I*iepssign) from a to infinity

Cplx Cqagpinf(Cplx (*f) (Double),Double a,Double singe,int iepssign,
              Double epsrel,Double *error,Double limit1,Double limit2);

Cplx Cqagpinfasqrtsing(Cplx (*f) (Double),Double a,Double singe,int iepssign,
                       Double epsabs,Double *error);

Cplx Cqagp2inf(Cplx (*f) (Double),Cplx (*Df) (Double),
               Double a,Double singe,int iepssign,
               Double epsrel,Double *error);
       // integration of f(s)/(s-singe-iepssign*I*eps)^2 to infinity
       // Df(x) is supposed to be the derivative of f(x) 

Cplx Cqagp2inf(Cplx (*f) (Double),Cplx (*Df) (Double),
               Double a,Double singe,int iepssign,
               Double epsrel,Double *error,Double limit1,Double limit2);
       // integration of f(s)/(s-singe-iepssign*I*eps)^2 to infinity
       // Df(x) is supposed to be the derivative of f(x) 


Cplx Cqagp2(Cplx (*f) (Double),Cplx (*Df) (Double),
            Double a,Double b,Double singe,int iepssign,
            Double epsrel,Double *error);
    // calculation of principal value
    // of f(x)/(x-singe)^2 in the interval from a to b 
